#!/bin/bash

set -eu

# CONFIGURATION

PRECOMP_WIN=/cygdrive/c/bin/precomp.exe
ZPAQ_WIN=/cygdrive/c/bin/zpaq64.exe

PRECOMPFLAGS="-cn"
ZPAQFLAGS="-method 3"
PARALLELISM=6

# ---

SELF_EXECUTABLE_NAME="$0"

trash() {
	if ! [[ -d ./.trashes ]] ; then
		mkdir ./.trashes 2>/dev/null || true
	fi
	
	mv "$1" ./.trashes/
}

addsuffix() {
	local infile="$1"
	local suffix="$2"
	local ret="${infile}${suffix}"
	while true ; do
		if [[ -f "$ret" ]] ; then
			ret="${ret}-$(date +%s)${suffix}"
		else
			break
		fi
	done
	echo "$ret"
}

precomp_filter() {
	local target=""
	
	while read -r infile ; do
	
		target="$(addsuffix "${infile}" ".pcf")"
		
		echo "precomp: ${infile}" >&2
		
		local tmpdir="$(mktemp -d /tmp/assimilate-precomp.XXXXXXXXXX)"
			
		(
			local cwd="$(pwd)"			
			cd "$tmpdir"			
			"$PRECOMP_WIN" $PRECOMPFLAGS -o"$(cygpath -w "$cwd/$target")" "$(cygpath -w "$cwd/$infile")"
		) >&2
		
		rm -r "$tmpdir"
		trash "$infile"
	
		# Pass on to the next filter
	
		echo "$target"
	
	done
}

zpaq_sink() {
	local zpaq_target="$1"

	while read -r precomped ; do
	
		echo "zpaq: ${precomped}" >&2
	
		"$ZPAQ_WIN" add "$(cygpath -w "$zpaq_target")" "$(cygpath -w "$precomped")" $ZPAQFLAGS
	
		trash "$precomped"
	
	done
}

file_source() {
	ls | grep -v '\.zpaq$' | grep -v '\.sh$'
}

get_zpaq_target() {		
	local num_zpaqs=$(ls | egrep '\.zpaq$' | wc -l)
	local filename=""

	if [[ "$num_zpaqs" -eq "0" ]] ; then
		filename="$(date +%s).zpaq"
		echo "INFO: creating ${filename} file" >&2
		touch "./${filename}"

	elif [[ "$num_zpaqs" -ne "1" ]] ; then
		echo "FATAL: More than one zpaq file, not sure what to do"
		exit 1
		
	else
		filename="$(ls | egrep '\.zpaq$')"
		
	fi
	
	echo "$filename"
}

assimilate_main() {
	# Main entry point		
	local dest="$(get_zpaq_target)"
	
	# Buffer the file list, so we don't pack our own temp files
	local file_list="$(file_source)"

	echo "$file_list" \
		| parallel --progress --max-procs="$PARALLELISM" "$SELF_EXECUTABLE_NAME --child" \
		| zpaq_sink "$dest"
}

usage() {
	cat <<'EOD'
Usage:
  assimilate [options]
  
Options:
  {no option argument}  Begin interactive pack/unpack workflow
  --child 'arg'         (Internal use only)
  --help                Display this message
  --pack                Immediately pack
  --unpack-menu         Begin interactive unpack workflow

EOD
	exit 1
}

build_dialog_command() {
	local zpfile="$(get_zpaq_target)"
	local packedfile=''
	
	echo -n 'dialog --stderr --no-tags --menu "Select file to unpack" 20 70 18'
	( "$ZPAQ_WIN" list "$(cygpath -w "$zpfile")" 2>&1 ) | grep '^-' | tr -s ' ' | cut -d' ' -f6- | while read -r packedfile ; do
		echo -n " \"$packedfile\" \"$packedfile\""
	done
	echo ""
}

unpack_menu() {
	local zpfile="$(get_zpaq_target)"
	local dialogres_file="$(mktemp)"
	
	(echo "$(build_dialog_command)" | bash -s ) 2>"$dialogres_file"
	
	local dialogres="$(cat "$dialogres_file")"
	
	rm "$dialogres_file"
	
	if [[ -z $dialogres ]] ; then
		echo "FATAL: No file selected" >&2
		exit 1
	else
		"$ZPAQ_WIN" extract "$(cygpath -w "$zpfile")" "$dialogres"
		"$PRECOMP_WIN" -r "$dialogres"
	
	fi
}

main_menu() {
	if dialog --yes-label "Pack" --no-label "Unpack" --yesno "Select mode" 8 70 ; then
		assimilate_main
	else
		unpack_menu
	fi
}

main() {
	if [[ "$#" -eq 0 ]] ; then
		main_menu
		read -p "Press any key to continue..."
	
	elif [[ "$#" -eq 2 && "$1" == "--child" ]] ; then
		# We're a parallel child
		echo "$2" | precomp_filter
	
	else
		case "$1" in
			--pack)
				assimilate_main
			;;
			--unpack-menu)
				unpack_menu
				read -p "Press any key to continue..."
			;;
			*)
				usage
			;;
		esac
	fi
}

main "$@"
